#include <Python.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// 调用 Python 函数
char **call_install_package(char **package_names, int package_count, const char *rootpath, const char *rpmdb_path, int *result_count) {
    PyObject *pName, *pModule, *pFunc;
    PyObject *pArgs, *pValue;
    char **result = NULL;

    // 初始化 Python 解释器
    Py_Initialize();

    // 添加当前目录到 Python 模块搜索路径
    PyRun_SimpleString("import sys");
    PyRun_SimpleString("sys.path.append('.')");

    // 加载 Python 脚本
    pName = PyUnicode_DecodeFSDefault("dnf_manager");
    pModule = PyImport_Import(pName);
    Py_DECREF(pName);

    if (pModule != NULL) {
        // 获取函数
        pFunc = PyObject_GetAttrString(pModule, "install_package");
        if (pFunc && PyCallable_Check(pFunc)) {
            // 构造参数
            pArgs = PyTuple_New(3);
            PyObject *pList = PyList_New(package_count);
            for (int i = 0; i < package_count; ++i) {
                pValue = PyUnicode_FromString(package_names[i]);
                if (!pValue) {
                    Py_DECREF(pArgs);
                    Py_DECREF(pModule);
                    fprintf(stderr, "Cannot convert argument\n");
                    return NULL;
                }
                PyList_SetItem(pList, i, pValue);
            }
            PyTuple_SetItem(pArgs, 0, pList);
            PyTuple_SetItem(pArgs, 1, PyUnicode_FromString(rootpath));
            PyTuple_SetItem(pArgs, 2, PyUnicode_FromString(rpmdb_path));

            // 调用函数
            pValue = PyObject_CallObject(pFunc, pArgs);
            Py_DECREF(pArgs);
            if (pValue != NULL) {
                if (PyList_Check(pValue)) {
                    *result_count = PyList_Size(pValue);
                    result = malloc(*result_count * sizeof(char *));
                    for (int i = 0; i < *result_count; ++i) {
                        PyObject *pItem = PyList_GetItem(pValue, i);
                        result[i] = strdup(PyUnicode_AsUTF8(pItem));
                    }
                }
                Py_DECREF(pValue);
            } else {
                Py_DECREF(pFunc);
                Py_DECREF(pModule);
                PyErr_Print();
                fprintf(stderr, "Call failed\n");
                return NULL;
            }
        } else {
            if (PyErr_Occurred())
                PyErr_Print();
            fprintf(stderr, "Cannot find function \"install_package\"\n");
        }
        Py_XDECREF(pFunc);
        Py_DECREF(pModule);
    } else {
        PyErr_Print();
        fprintf(stderr, "Failed to load \"dnf_manager\"\n");
        return NULL;
    }

    // 结束 Python 解释器
    Py_Finalize();

    return result;
}

void call_query_package(char **package_names, int package_count) {
    PyObject *pName, *pModule, *pFunc;
    PyObject *pArgs, *pList;

    Py_Initialize();
    pName = PyUnicode_DecodeFSDefault("dnf_manager");
    pModule = PyImport_Import(pName);
    Py_DECREF(pName);

    if (pModule != NULL) {
        pFunc = PyObject_GetAttrString(pModule, "query_package");
        if (pFunc && PyCallable_Check(pFunc)) {
            pList = PyList_New(package_count);
            for (int i = 0; i < package_count; ++i) {
                PyObject *pValue = PyUnicode_FromString(package_names[i]);
                if (!pValue) {
                    Py_DECREF(pList);
                    Py_DECREF(pModule);
                    PyErr_Print();
                    fprintf(stderr, "Cannot convert argument\n");
                    return;
                }
                PyList_SetItem(pList, i, pValue);
            }

            pArgs = PyTuple_Pack(1, pList);
            Py_DECREF(pList);

            if (pArgs != NULL) {
                PyObject *pResult = PyObject_CallObject(pFunc, pArgs);
                Py_DECREF(pArgs);
                if (pResult != NULL) {
                    Py_DECREF(pResult);
                } else {
                    PyErr_Print();
                    fprintf(stderr, "Call to query_package failed\n");
                }
            } else {
                PyErr_Print();
                fprintf(stderr, "Cannot create argument tuple\n");
            }
        } else {
            if (PyErr_Occurred())
                PyErr_Print();
            fprintf(stderr, "Cannot find function 'query_package'\n");
        }
        Py_XDECREF(pFunc);
        Py_DECREF(pModule);
    } else {
        PyErr_Print();
        fprintf(stderr, "Failed to load 'dnf_manager'\n");
    }
    Py_Finalize();
}